# Copyrights 2001-2014 by [Mark Overmeer].
#  For other contributors see ChangeLog.
# See the manual pages for details on the licensing terms.
# Pod stripped from pm file by OODoc 2.01.

use strict;
use warnings;

package Mail::Box::MH::Message;
our $VERSION = '2.111';

use base 'Mail::Box::Dir::Message';

use File::Copy;
use Carp;


#-------------------------------------------


# Purpose of doc is only the warning... no new implementation required.

#-------------------------------------------


1;
