# Copyrights 2001-2014 by [Mark Overmeer].
#  For other contributors see ChangeLog.
# See the manual pages for details on the licensing terms.
# Pod stripped from pm file by OODoc 2.01.

package MailBox::Test::31fgroups::Definition;
our $VERSION = '2.111';


sub name     {"Mail::Message::Head::FieldGroup; grouping fields"}
sub critical {0}
sub skip     { undef }

1;
